export default function ComingSoon() {
  return (
    <div className="not-content coming-soon-banner">
      <span className="coming-soon-badge">New</span>
      <h3>Expanded Recipe Registries & Benchmarking</h3>
      <p>
        Community-driven recipe registries and integrated benchmarking support are now available
        — powered by <a href="https://spark-arena.com/" target="_blank" rel="noopener noreferrer">Spark Arena</a>.
      </p>
    </div>
  );
}
